/*
 epiclccorr

   Authors:  Aitor Ibarra - XMM-Newton SOC ESAC/ESA

   Description: The aim of this task is to correct EPIC source time series.

   Comments:

  $Id: epiclccorr.cc,v 1.8 2011/02/17 15:46:26 aibarra Exp $
*/

#include <vector>
#include <string>
#include "epiclccorr.h"
#include "LightCurve.h"
#include <Tracer.h>
#include <errstr.h>


void epiclccorr ()
{
PROC
	/*
	 * List of the source time series
	 */
	string srcTSList = stringParameter("srctslist");

	/*
	 * List of the original event files
	 */
	string evtList = stringParameter("eventlist");

	/*
	 * Output name
	 */
	string outFileName = stringParameter("outset");


	/*
	 * Creates the source Light Curve
	 */
	LightCurve *srcLC = new LightCurve(srcTSList, evtList);


	/*
	 * Apply relative corrections
	 */

	srcLC->doRelativeCorr(Spect::SRC);

	/*
	 * Calculates BACKSCALE value for source time serie
	 */
	if ( ( booleanParameter("withbkgset") ) ||
			(booleanParameter("applyabsolutecorrections")) )
	{
		srcLC->calSpec();
		// arfgen modifies the spectrum with the correct backscale.
		srcLC->getSpec()->calArfgen(Spect::SRC);
		srcLC->getSpec()->readSpec();
	}

	/*
	 * Check if we have to apply absolute corrections
	 */
	if (booleanParameter("applyabsolutecorrections"))
	{
		srcLC->getSpec()->readArf(Spect::SRC);
		srcLC->getSpec()->calArfgen(Spect::ON);
		srcLC->getSpec()->readArf(Spect::ON);
		srcLC->doAbsoluteCorr();
	}
	// Remove src ancillary file
	if ( ( booleanParameter("withbkgset") ) ||
			(booleanParameter("applyabsolutecorrections")) )
		srcLC->getSpec()->removeArfFile();


	srcLC->getLC()->calRateAndErr();

	/*
	 * Check if a bkg light curve have been supplied and create the
	 * corresponding light curve.
	 * Calculate relative corrections and absolute corrections for background
	 * time serie
	 */
	LightCurve *bkgLC = 0;
	if (booleanParameter("withbkgset"))
	{
		string bkgTSName = stringParameter("bkgtslist");
		bkgLC = new LightCurve(bkgTSName, evtList);

		bkgLC->doRelativeCorr(Spect::BKG);

		/*
		 * Calculation of  BACKSCALE
		 */
		bkgLC->calSpec();
		bkgLC->getSpec()->calArfgen(Spect::BKG);
		bkgLC->getSpec()->readSpec();
		bkgLC->getLC()->calRateAndErr();
		// Remove the bkg arf file
		bkgLC->getSpec()->removeBkgArfFile();


		/*
		 * Substract background:
		 * If the src lightcurve falls in more than one CCD,
		 * we have to substract the background for each CCD.
		 * and calculate the rate again...
		 */
		if (srcLC->moreThanOneCCD())
		{
			// Calculate the source background subtracted counts per CCD
			srcLC->substractBKGForAllCCDs(bkgLC);
			// Calculate the new count rates.
			srcLC->getLC()->calRateAndErr();
		}
		else
		  srcLC = srcLC->operator-(bkgLC);
	}



	if (booleanParameter("applyabsolutecorrections"))
		srcLC->applyAbsCorr();


	srcLC->createFile(outFileName, bkgLC);

	delete srcLC;
	delete bkgLC;


ENDPROC
}
